#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <math.h>

#include "v4.h"

#ifdef NEED_POW2
static double pow2(double parm)
{
	return pow(2, parm);
}
#endif

#define PRE_SINUS 0 /* use precalculated sinus values? */

#define X_MAX     2048

#if PRE_SINUS
 #define SINUS_MAX (2 << 16)
#else
 #define SINUS_MAX (M_PI + M_PI)
#endif

#define PlaybackRate 11025

static int Scales[X_MAX];
static float Fq[X_MAX], FqAdd[X_MAX];

static int MARGIN, WIDTH;

#if PRE_SINUS
static float Sinus[SINUS_MAX];
#endif

static void Init(void)
{
	float Min, Max, MaxMinDiff;

	int a;

	unsigned char *Pal;
	
	MARGIN = (COL - 140) / 2;
	
	/* Draw the white lines */
//	memset(&Vid[(LINE-1)*COL], 255, COL);	
	memset(&Vid[V4_TOP*COL], 255, COL);
	
	/* Set the palette */
	for(a=16; a<136; a++)
	{
		Pal = &Palette[a*3];	/* 16..135 = 0x10..0x87 */

		*Pal++ = (a<0x60) ? 0 : (a-0x60) * 255 / 0x27;
		*Pal++ = (a<0x37) ? 0 : (a-0x37) * 255 / 0x50;
		*Pal   = a * 255 / 135;
		
		Pal += 120*3 - 2;

		*Pal++ = a * 255 / 135;
		*Pal++ = (a<0x37) ? 0 : (a-0x37) * 255 / 0x50;
		*Pal   = (a<0x60) ? 0 : (a-0x60) * 255 / 0x27;
	}
	
	fb_writepalette();
	
	Min = log(V4_MIN) / log(2.0);
	Max = log(V4_MAX) / log(2.0);
	
	MaxMinDiff = Max-Min;
	
	WIDTH = COL-MARGIN*2;
	
	for(a=0; a<WIDTH; a++)
	{
		Fq[a] = 0.0;
		Scales[a] = pow2(a * MaxMinDiff / WIDTH + Min);
		FqAdd[a] = Scales[a] * FREQ * 2.0 * SINUS_MAX / N / PlaybackRate;
	}
	
#if PRE_SINUS
	for(a=0; a<SINUS_MAX; a++)
		Sinus[a] = sin(a * 2*M_PI / SINUS_MAX);
#endif
}

static void Done(void)
{
}

static void View(void)
{
	char fps_str[32];
	unsigned char *Ptr;
	float Data[X_MAX], W80;
	int a, b;
	float n;
	
	for(a=0; a<WIDTH; a++)
	{
		register int nyt = Scales[a];
		float temp = Power[nyt] - Avg[nyt]/AvgCount;
		Data[a] = temp<0.0 ? 0.0 : sqrt(temp);
	}
	
	sprintf(fps_str, "FPS=%.1f numf=%d (%d..%d)", FPS, WIDTH, (int)(V4_MINHERZ), (int)(V4_MAXHERZ));
	fb_center(0, fps_str);
	
	Ptr = Vid + COL * (LINE-2) + MARGIN;
	
	memmove(
		&Vid[(V4_TOP+1)*COL], 
		&Vid[(V4_TOP+2)*COL], 
		(LINE-V4_TOP-2)*COL);
	
	*Ptr++ = 255;	
	
	W80 = 80.0 / WIDTH;
	
	for(b=0; b<PlaybackRate/50; b++)
	{
		n = 128.0;
		
		for(a=0; a<WIDTH; a++)
		{
			register float dy = Data[a];
			
			if(!b)
			{
				register int Dy = dy * 20.0;
				if(Dy>135)Dy=135;
				if(Dy<16) Dy=16;
				
				*Ptr++ = Dy;
			}
			
#if PRE_SINUS
			n += dy*W80 * Sinus[(int)Fq[a] & (SINUS_MAX-1)];
#else
			n += dy*W80 * sin(Fq[a]);
#endif
			
			Fq[a] += FqAdd[a];
		}
		
//		if(n<0)n=0;
//		if(n>255)n=255;
		
		putchar((int)n);
	}
	
	*Ptr = 255;
}

Viewer V4 = {fb_is_required, 0, Init, Done, View};
